/*
 * Macro to parse analytic report configuration file
 *
 * Author: Song, Qiang <qiang.song@sas.com> 2014
 *
 */

%macro rptcfg_extract_param_dst(xmlfile =, out = , type = );
/* DOCSSTRING
#input
- xmlfile: path to the location of the XML configuration file; do not include quotations
- type: expecting either daily, weekly or monthly

#output
- out: a SAS dataset contains the parameters to generate reports
   END DOCSTRING */


%local outdst outlib;

%let outlib = %scan(&out, 1, .);
%let outdst = %scan(&out, 2, .);
%let outlib = %sysfunc(ifc("&outdst" = "", work, &outlib));
%let outdst = %sysfunc(ifc("&outdst" = "", &out, &outdst));

data _temp_out_(drop = type drill_down_list);
    if _N_ = 1 then do;
        length
            drl_name        $32
            drl_var         $32
            drl_format      $34
            drl_label       $500
            ;
        drop
            rc
            drl_name
            drl_var
            drl_format
            drl_label
            ;

        declare hash drills(dataset: "rptcfg.drill_down_list");
        drills.definekey("drl_name");
        drills.definedata("drl_var", "drl_format", "drl_label");
        drills.definedone();

        call missing(of drl_:);
    end;

    set rptcfg.reports(where = (type in  ("all", "&type")));

    if missing(drill_down_list) then do;
        call missing(of drill_down_:);
        output;
    end;
    else do;
        drop i;
        i = 1;
        drl_name = scan(drill_down_list, i);
        do while (not missing(drl_name));
            rc = drills.find();
            if rc = 0 then do;
                drill_down_variable = drl_var;
                drill_down_format = drl_format;
                drill_down_label = drl_label;
                output;
            end;
            else do;
                putlog "ERROR: missing drill_down tyle specification for " drl_name;
            end;
            i = i + 1;
            drl_name = scan(drill_down_list, i);
        end;
    end;
run;

proc sql noprint;
    create table &out like _temp_out_;
quit;

/* data validation */
proc datasets  library = &outlib nolist;
    modify &outdst;

    ic create val_report_variable =
        not null(report_variable)
        message="Report_variable cannot be null";

    ic create val_percentile_report =
        check(where = (percentile_report in ("Y", "N")))
        message = "Indicator percentile_report must be Y or N";

    ic create val_aggregation_type =
        check(where = (aggregation_type in ("N", "G", "R")))
        message = "aggregation_type be N (no aggregation), G (group by) or R (group by then aggregation)";

    ic create val_aggregation =
        check(where = (aggregation_type in ("N") and missing(aggregation_variable)
                       and missing(aggregation_function) and missing(aggregation_result_format)
                       and missing(aggregation_result_label)
                       or
                       aggregation_type in ("G", "R") and not missing(aggregation_variable)
                       and not missing(aggregation_function) and not missing(aggregation_result_format)
                       and not missing(aggregation_result_label)))
        message = "aggreation var, function, format, label must exist if aggregation is wanted";


    ic create val_stability_index =
        check(where = (stability_index in ("P", "D", "N")))
        message = "stability_index must be N (no SI), P(wrt previous period), or D(wrt development data)";
run;
quit;

proc append base=&out data=_temp_out_;
run;

proc datasets nolist library = work;
    delete _temp_out_;
run;
quit;

%mend rptcfg_extract_param_dst;
